﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Random;
using MathNet.Numerics.Statistics;
using DoubleVector = MathNet.Numerics.LinearAlgebra.Vector<double>;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class AzzaliBenchmark3 : ArtificialRegressionDataDescriptor {
    public override string Name { get { return "Azzali Benchmark3 B3 = CumMin[3,3](X1) * (X2 / X3) + X4"; } }
    public override string Description { get { return "I. Azzali, L. Vanneschi, S. Silva, I. Bakurov, and M. Giacobini, “A Vectorial Approach to Genetic Programming,” EuroGP, pp. 213–227, 2019."; } }

    protected override string TargetVariable { get { return "B3"; } }
    protected override string[] VariableNames { get { return AllowedInputVariables.Concat(new[] { TargetVariable }).ToArray(); } }
    protected override string[] AllowedInputVariables { get { return new string[] { "X1", "X2", "X3", "X4", "X5" }; } }
    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return 70; } }
    protected override int TestPartitionStart { get { return 70; } }
    protected override int TestPartitionEnd { get { return 100; } }

    public int Seed { get; private set; }

    public AzzaliBenchmark3()
      : this((int)DateTime.Now.Ticks) { }
    public AzzaliBenchmark3(int seed)
      : base() {
      Seed = seed;
    }


    protected override List<List<double>> GenerateValues() { return null; }
    protected override List<IList> GenerateValuesExtended() {
      var rand = new MersenneTwister((uint)Seed);

      var x1Column = new List<DoubleVector>(100);
      var x2Column = new List<double>(100);
      var x3Column = new List<double>(100);
      var x4Column = new List<double>(100);
      var x5Column = new List<double>(100);
      var b3Column = new List<DoubleVector>(100);

      for (int i = 0; i < 100; i++) {
        var x1 = rand.NextDoubleVector(10, 30, 20);
        var x2 = rand.NextDouble(50, 60);
        var x3 = rand.NextDouble(5, 10);
        var x4 = rand.NextDouble(-2, +2);
        var x5 = rand.NextDouble(0, 1);

        const int p = 3, q = 3;
        var cumulativeMin = DoubleVector.Build.DenseOfEnumerable(
          Enumerable.Range(0, x1.Count)
            .Select(idx => Enumerable.Range(idx - p, q)) // build index ranges for each target entry
            .Select(indices => indices.Select(idx => idx < 0 ? 0 : x1[idx])) // take the values from the vector (fill with 0 if outside)
            .Select(values => values.Min())
          );
        var b3 = (cumulativeMin * (x2 / x3)) + x4;

        x1Column.Add(x1);
        x2Column.Add(x2);
        x3Column.Add(x3);
        x4Column.Add(x4);
        x5Column.Add(x5);
        b3Column.Add(b3);
      }

      return new List<IList> { x1Column, x2Column, x3Column, x4Column, x5Column, b3Column };
    }
  }
}