﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Problems.DataAnalysis;

using DoubleVector = MathNet.Numerics.LinearAlgebra.Vector<double>;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public static class RandomExtensions {
    public static double NextDouble(this IRandom rand, double min, double max) {
      return (max - min) * rand.NextDouble() + min;
    }
    public static DoubleVector NextDoubleVector(this IRandom rand, double min, double max, int length) {
      var values = new double[length];
      for (int i = 0; i < values.Length; i++) {
        values[i] = rand.NextDouble(min, max);
      }
      return DoubleVector.Build.Dense(values);
    }

    public static DoubleVector NextIncreasingDoubleVector(this IRandom rand, double min, double max, int length) {
      var values = new double[length];
      for (int i = 0; i < values.Length; i++) {
        double offset = i * rand.NextDouble();
        values[i] = rand.NextDouble(min, max) + offset;
      }
      return DoubleVector.Build.Dense(values);
    }
  }
}