﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class VectorDataInstanceProvider : ArtificialRegressionInstanceProvider {
    public override string Name {
      get { return "Vector Data Benchmark Problems"; }
    }
    public override string Description {
      get { return ""; }
    }
    public override Uri WebLink {
      get { return new Uri("http://dev.heuristiclab.com/trac/hl/core/wiki/AdditionalMaterial"); }
    }
    public override string ReferencePublication {
      get { return ""; }
    }
    public int Seed { get; private set; }

    public VectorDataInstanceProvider()
      : this((int)DateTime.Now.Ticks) { }

    public VectorDataInstanceProvider(int seed)
      : base() {
      Seed = seed;
    }
    public override IEnumerable<IDataDescriptor> GetDataDescriptors() {
      var rand = new MersenneTwister((uint)Seed);
      return new List<IDataDescriptor> {
        new VectorDataTestOneA(rand.Next()),
        new VectorDataTestOneB(rand.Next()),
        new VectorDataTestOneC(rand.Next()),
        new VectorDataTestOneD(rand.Next()),
        new VectorDataTestTwoA(rand.Next()),
        new VectorDataTestTwoB(rand.Next()),
        //new VectorDataTestTwoC(rand.Next()),
        //new VectorDataTestTwoD(rand.Next()),
        new VectorDataTestThreeA(rand.Next()),
        new VectorDataTestThreeB(rand.Next()),
        new VectorDataTestThreeC(rand.Next()),
        new VectorDataTestThreeD(rand.Next()),
        new VectorDataTestFourA(rand.Next()),
        new VectorDataTestFourB(rand.Next()),
        new VectorDataTestFourC(rand.Next()),
        new VectorDataTestFourD(rand.Next()),
        new AzzaliBenchmark1(rand.Next()),
        new AzzaliBenchmark2(rand.Next()),
        new AzzaliBenchmark3(rand.Next()),
        new AzzaliKorns5(rand.Next())
      };
    }
  }
}
